/*
** libdebug.h for Project in /home/phil/Travail/Thales/hookconnector/code/nfhcctl/src/lib/debug
**
** Made by thierr_p
** Login   <philippe.THIERRY@fr.thalesgroup.com>
**
** Started on  ven 22 ao 2008 16:58:22 CET thierr_p
** Last update ven. 17 juil. 2009 20:57:54 CEST phil
*/
/*
** Copyright (C) 2009 - Philippe THIERRY
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License
** as published by the Free Software Foundation; either version 2
** of the License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#ifndef LIBDEBUG_H_
# define LIBDEBUG_H_

#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <inttypes.h>
#include <ctype.h>
#include <sys/types.h>
#include <stdint.h>
#include <string.h>
#include <signal.h>
#include <time.h>
#include <syslog.h>

#include "libutils.h"
#include "libmemtrace.h"

/*!
** \file libdebug.h is the interface of the debug library.
**
** This library implements different capacities of debugging, warning,
** erroring and panic mechanisms.
*/

#define MAX_PREFIX_SIZE		128

/*!
** @brief This enum describes the different levels of debugging.
**
** The different levels of debbuging are supported in order to manage
** different types of debbuging techniques.
**
** Each function warning, erroring and debugging manage a counter and a time
** management in order to be well informed of the execution time of each of
** them, and the number of each warning, error and debug call.
*/
typedef enum
{
  DEBUG_PANIC = 0, /* system is unusable */
  DEBUG_ALERT,     /* action must be taken immediatly*/
  DEBUG_CRIT,      /* critical conditions */
  DEBUG_ERROR,     /* error conditions*/
  DEBUG_WARNING,   /* warning conditions */
  DEBUG_NOTICE,    /* normal but signifiant notice */
  DEBUG_INFO,      /* Simple informational */
  DEBUG_DEBUG,     /* debug-level message */
  DEBUG_NONE,      /* no messages */
  DEBUG_MAX        /* max debug value. First invalid */
} e_debugmode;

typedef enum
{
  DEBUG_TYPE_STDERR = 0,
  DEBUG_TYPE_SYSLOG,
  DEBUG_TYPE_MAX
} e_debugtype;

/*
** CAUTION: This file is the libdebug interface. It should be used by whatever
** software using this lib.
*/

/*!
** @brief get the current debug mode.
**
** Return the current debug mode. Used to check before debug message.
**
** @return the current debug mode.
*/
uint8_t
cm_debug_get_mode(void);


/*****************************************************************************
** Macro definitions
*****************************************************************************/

/*!
** \def Specify the return value of a panic exit
*/
#define	DEBUG_KILL	127

e_debugtype
cm_debug_get_type(void);

const char*
cm_debug_get_level_string(e_debugmode level);

const char*
cm_debug_get_prefix(void);

uint64_t
cm_debug_get_count(void);

/*!
** The DBG_TRACE macro is used for debugging information
**
** This macro automaticaly support syslog/stderr logging facilities, depending
** on the cm_debug_init() call.
**
** The explicit cast is due to printf incapacity to manage time_t types.
*/
#define DBG_TRACE(level, ...)					 \
  if (cm_debug_get_type() == DEBUG_TYPE_STDERR)			 \
  {								 \
    if (level >= cm_debug_get_mode())				 \
    {								 \
      fprintf(stderr, "%s %s [ %08llu ] [ %08lld ] %s : %d -- ", \
             cm_debug_get_prefix(),                              \
             cm_debug_get_level_string(level),                   \
	     cm_debug_get_count(),                               \
             (long long int)time(NULL),                          \
              __FILE__, __LINE__);                               \
      fprintf(stderr, __VA_ARGS__);                              \
      fprintf(stderr, "\n");                                     \
    }								 \
  }								 \
  if (cm_debug_get_type() == DEBUG_TYPE_SYSLOG)			 \
  {								 \
    if (level >= cm_debug_get_mode())				 \
    {								 \
      char	dbgtrace_string[512];				 \
      int	dbgtrace_len = 0;				 \
      dbgtrace_len = snprintf((char*)dbgtrace_string, 512,       \
               "%s [ %08llu ] [ %08lld ] %s : %d -- ",           \
               cm_debug_get_level_string(level),                 \
	       cm_debug_get_count(),                             \
               (long long int)time(NULL),                        \
                __FILE__, __LINE__);                             \
      dbgtrace_len = snprintf(&(dbgtrace_string[dbgtrace_len]),  \
                              512 - dbgtrace_len, \
                     __VA_ARGS__);                               \
      syslog(LOG_USER | cm_debug_get_mode(),                     \
             "%s\n", dbgtrace_string);                          \
    }								 \
  }

/*****************************************************************************
** Prototypes
*****************************************************************************/

/*!
** @brief Print a panic and exit.
**
** When calling it, replace LINE with gcc standard __LINE__ value and replace
** FILENAME with gcc standard __file__ value.
**
** If libdebug has been initialized with a sighandler, the sighandler is
** called first before leaving.
**
** @param LINE line of code on which this function is called
** @param FILENAME the corresponding source file on which this function is called
** @param str the string to print
**
*/
void
panic(uint16_t		LINE,
      const char	*FILENAME,
      const char	*str,
      ...);

/*!
** @brief set the current debug mode.
**
** Define dynamicly the current debug mode. This allow the modification of the
** debug level during the program execution, instead of recalling the
** cm_debug_init function, which should be called only one time.
**
** @return the updated debug mode.
*/
uint8_t
cm_debug_set_mode(e_debugmode	mode);

/*!
 ** @brief initialize the debug manager.
 **
 ** This function should be called in order to specify the debug level and
 ** the debug output type (stderr or syslog)
 ** 
 ** @param level	the debug level you want, from DEBUG_NONE to DEBUG_DEBUG 
 ** @param dbgtype 	the debug output you want : stderr or syslog facility
 **
 ** @return 0 if init OK, -1 otherwise
 */
int8_t
cm_debug_init(e_debugmode		level,
              e_debugtype		dbgtype,
              const char * const	prefix);

/*!
 ** @brief release the debug manager.
 */
void
cm_debug_release(void);

#endif /* !LIBDEBUG_H_ */
