/*
** libutils.h for Project in /home/phil/Travail/TC11/find/trunk/src/utils
**
** Made by thierr_p
** Login   <philippe.THIERRY@fr.thalesgroup.com>
**
** Started on  lun 09 oct 2006 18:14:54 CEST thierr_p
** Last update sam. 04 juil. 2009 19:23:59 CEST phil
*/
/*
** Copyright (C) 2009 - Philippe THIERRY
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License
** as published by the Free Software Foundation; either version 2
** of the License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#ifndef LIBUTILS_H_
# define LIBUTILS_H_

# include <math.h>
# include <stdlib.h>
# include <unistd.h>
# include <fcntl.h>
# include <stdio.h>
# include <string.h>
# include <stdarg.h>
# include <limits.h>
# include <inttypes.h>

# include "libmemtrace.h"
# include "libstring.h"

# define MY_BUF_SIZE 512



/*!
** @brief Pointer union, useful to get pointer value in multiple format.
**
** @arg vp is a pointer on a void.
** @arg ul is an unsigned int64_t.
** @arg l is a int64_t.
** @arg cp is a pointer on char.
*/
union		u_pointer
{
  void		*vp;
  uint32_t	ul;
  int32_t	l;
  int8_t	*cp;
};

typedef union u_pointer	u_ptr;

/*!
** @brief This struct contains all datas for getln function.
** It should not be used only with getln.
**
** @arg fd is a file descriptor.
** @arg buf is a buffer which contains the lines.
** @arg cursor is used to know where is the \n in the buffer.
** @arg eof is used to know when the cursor is in the end.
*/
struct		buf
{
  int32_t		fd;
  char			buf[MY_BUF_SIZE];
  int32_t		cursor;
  int32_t		eof;
};

/*!
** \brief chomp delete the last character of str if it is a '\n' char
**
** Its behaviour corresponds to the perl chomp() function.
**
** @param str the string to modify
** @return the modified string
*/
char	*chomp(char	*str);

/*!
** \brief chop delete the last character of str, whatever it is
**
** Its behaviour corresponds to the perl chop() function.
**
** @param str the string to modify
** @return the modified string
*/
char	*chop(char	*str);

/****
** getln (get line) support
****/
typedef struct buf	s_buf;

s_buf	*alloc_getln_buf(int32_t	fd);

/*
** get a line from the given buffer, deleting it from the given buffer.
*/
char	*my_getln(s_buf	*buf);

/****
** end of getln
****/

/*!
** Allocation and desalocation stuff, in order to protect and add some
** capacities to memory allocation
** @{
*/

/*!
** \brief allocate dynamic memory, checking if malloc return is NULL first.
**
** If MEMTRACE is activated, use the memtrace facility
**
** @param s the size to allocate
** @return a valid pointer to the begining of the allocated memory field, or NULL
*/
void	*xmalloc(size_t	s);

/*!
** \brief reallocate dynamic memory, checking the realloc return value first.
**
** @param p the pointer to realloc
** @param s the new size
**
** @return a pointer to the begining of the new memory field, or NULL if the
** reallocation failed.
**
** The new memory field may start at the same place as the old one. If not,
** the old memory field is freed after having its content copied
*/
void	*xrealloc(void		*p,
                  size_t	s);

/*!
** \brief Free the given pointer, deleting it from memtrace facility if MEMTRACE is activated
**
** @param ptr the pointer to the memory field to free
*/
void	xfree(void	*ptr);


/*!
 ** @brief secure_malloc allow memory content protection
 **
 ** When using dynamic memory management, these functions manage memsetting
 ** when freeing memory, in order to avoid data sharing between process in
 ** not mastered environnements.
 **
 ** secure_malloc just associate a metadata list to pointers
 ** 
 ** @param s the size to allocate
 ** @return a pointer address to a secure memory
 */
void*
secure_malloc(size_t	s);

/*!
 ** @brief secure_free memset the given pointed address before freeing
 **
 ** The pointer must have been allocated with secure_malloc.
 ** 
 ** @param p the pointer to free "securely"
 */
void
secure_free(void *	p);

/*!
** }@
*/

/*!
** \brief Convert an int32_t onto a string
**
** @param val the numeric value to translate
**
** @return a string corresponding to the argument value
*/
char	*my_itoa(int32_t	val);

/*!
** \brief Convert a decmal value onto an octal value. CAUTION: Values are integer
**
** @param the decimal value to convert
**
** @return the octal value, seen as an integer (octal values only contain digits)
*/
int32_t	dec2oct(int32_t	dec);

/*!
** \brief return true if the string contains only digits (and may begins with '+' or ** '-')
*/
int32_t	is_number(const char	*val);

/*!
** \brief Return true if the char is a space or a tab.
*/
int32_t	is_space(char	c);

/*
** FIXME
*/
int32_t	is_end(char	c);

/*!
** \brief Lexer basic support.
** Return each word, considering each char in the 'list' param as a separator
**
** @param buf a string pointer on which the lexer executes its treatment
** @param list the list of separators
**
*/
char	*cm_lexer(char	**buf,
                  char	*list);

/*!
** \brief Return true if the current char is a digit.
** @param c the char to check
** @return true if the given char is between '0' and '9'
**
*/
int32_t	my_isdigit(char	c);

/*
** Print the given number to stdout.
*/
void	print_num(int32_t	i);

/*!
** \brief Convert the given value in a binary value (huge int32_t)
**
** Take care that the associated binary value is lower than INT_MAX.
**
** @return the binary value, written as an int
*/
int32_t	dec2bin(int32_t	dec);

/*!
** \brief return the pointer address has a string
**
** @param ptr the pointer we want to print the address of
**
** @return a string containing a "0xXXXXXXXX" value, corresponding to ptr
*/
char	*ptr2hex(void	*ptr);

/*!
** \brief Simple regex matching using pattern.
**
** Compare the given string to the given pattern
**
** Support basic regex :
** [<chars>], {<num>,<num>}, *, ?, +
**
** @param str the string to use
** @param pat the pattern to verify
**
** @return true if string and pattern correspond, false otherwise
*/
int32_t	cm_match(char	*str,
                 char	*pat);

/*!
** \brief convert big endian char into little endian
**
** @param c the char to convert
**
** @return the converted char
*/
char	big2little(char	c);

/*!
** \brief convert little endian char into big endian
**
** @param c the char to convert
**
** @return the converted char
*/
char	little2big(char	c);

/*!
** \brief return the number of active CPU on the current architecture
*/
uint16_t	cm_nrcpu(void);

#endif /* !LIBUTILS_H_ */
